# install.packages("shiny", dependencies = T)
# install.packages("shinythemes", dependencies = T)
# install.packages("TeachingDemos", dependencies = T)

library(shiny)
library(shinythemes)
library(TeachingDemos)


ui <- shinyUI(navbarPage(theme =shinytheme("darkly"),"PROBABILIDAD",
                         tabPanel("PROBABILIDAD",uiOutput("component1")),
                         tabPanel("BINOMIAL",uiOutput("component2")),
                         tabPanel("NORMAL", uiOutput("component3")),
                         tabPanel("APROXIMACIÓN BINOMIAL A NORMAL", uiOutput("component4"))
))

server <- function(input, output) {
  
  ########################### PROBABILIDAD -- COMPONENTE 1 ##########################################################
  
  output$component1 <- renderUI({
    sidebarLayout(
      sidebarPanel(width = 0,
                   fluidRow(column(8, br(),br(),br(),h1("PROBABILIDAD")),
                            column(4,img(src = "probabilidad.gif", height = 200, width = 200)))
                   ,br(),br()
      ),
      mainPanel(width = 12,
        wellPanel(style = "background: grey",
                  h3("Contenidos"),br(),
                  h4(strong("Esta aplicación contiene las siguientes ventanas:")),
                  h5(strong("--  Sección Binomial: "),em("distribución de probabilidad discreta, mide el número de éxitos en una secuencia de n ensayos independientes, con una probabilidad fija p de ocurrencia de éxitos entre los ensayos")),
                  h5(strong("--  Sección Normal: "),em("distribución la más importante y de mayor uso de las distribuciones continuas, debido a la gran cantidad de fenómenos aleatorios que modeliza. Identificada por dos parámetros que coinciden con su media y su desviación típica")),
                  h5(strong("--  Sección Aproximación Binomial por la Normal: "),em("una distribución Binomial con valores de los parámetros grandes puede aproximar bien a una distribución Normal")),
                  br(),
                  h3("Simulación tiradas de dados"),br(),
                  fluidRow(column(5, numericInput(inputId = "ndice", label = "Número de dados a tirar", min = 0, value = 4),
                                  numericInput(inputId = "rolls", label = "Número de veces que se tiran los dados", min = 0, value = 8) )) ,
                  fluidRow(column(9,plotOutput("graficoDados")),
                           column(3,p("Medias de los puntos obtenidos en cada dado:")
                                  ,verbatimTextOutput("mediaDados")))
                  
        )
      )
    )
  })
  
  ########################### BINOMIAL -- COMPONENTE 2 ##########################################################
  
  output$component2 <- renderUI({
    sidebarLayout(
      sidebarPanel(
        wellPanel(style = "background: grey",
                  h4("Distribución binomial"),
                  p("Parámetros de la distribución binomial: "),
                  numericInput(inputId = "n", label = "Número de repeticiones, n", min = 0, value = 10),
                  numericInput(inputId = "p",label = "Probabilidad de éxito, p",min = 0,max = 1, value = 0.1, step = 0.05),
                  br(),
                  img(src = "binomial.gif", height = 200, width = 200), br(),br(),
                  p("Variable X -> B(n,p)"),
                  p("Función de probabilidad, P(X = x):",em(" probabilidad de que la variable aleatoria X tome exactamente el valor x")),
                  p("Función de distribución, P(X <= x):",em(" probabilidad de que una variable aleatoria X asuma un valor inferior o igual a x"))

        )  
      ),
      
      mainPanel(
        wellPanel(style= "background:grey",
                  h3("La distribución binomial"),br(),
                  
                  p("Gráfico de la función de probabilidad"),
                  plotOutput("graficoBinomial"),br(),
                  
                  fluidRow(column(5,strong("Esperanza: "),
                                  verbatimTextOutput("esperanza")),
                           column(5,strong("Varianza: "),
                                  verbatimTextOutput("varianza"))),
                  
                  fluidRow(column(6,br(),strong("Probabilidad de obtener exactamente ")),
                           column(3, numericInput(inputId = "exitos",label = " ", value = 0,min = 0)),
                           column(1,br(),strong(" éxitos"))
                  ),
                  verbatimTextOutput("probabilidad"),br(),
                  
                  fluidRow(column(6,br(),strong("Probabilidad de obtener al menos ")),
                           column(3, numericInput(inputId = "exitos1",label = " ", value = 0,min = 0)),
                           column(1,br(),strong(" éxitos"))
                  ),
                  verbatimTextOutput("probabilidadAlMenos"),br(),
                  
                  fluidRow(column(6,br(),strong("Probabilidad de obtener un maximo de ")),
                           column(3, numericInput(inputId = "exitos2",label = " ", value = 0,min = 0)),
                           column(1,br(),strong(" éxitos"))
                  ),
                  verbatimTextOutput("probabilidadMucho"),br(),
                  
                  fluidRow(column(5,br(),strong("Probabilidad de obtener entre ")),
                           column(2, numericInput(inputId = "exitosP",label = " ", value = 0,min = 0)),
                           column(1,br(),strong(" y")),
                           column(2, numericInput(inputId = "exitosS",label = " ", value = 1,min = 0)),
                           column(1,br(),strong(" éxitos"))
                  ),
                  verbatimTextOutput("probabilidadIntervalo"),br(),
                  
                  fluidRow(column(6,br(),strong("Valor de la variable que deja a su derecha la probabilidad de ")),
                           column(2, numericInput(inputId = "porcentaje",label = " ", value = 0,min = 0, max = 1,step = 0.05))
                  ),
                  verbatimTextOutput("PorcentajeObs"),br(),
                  
                  fluidRow(column(6,br(),strong("Generación de una muestra aleatoria de tamaño ")),
                           column(2, numericInput(inputId = "valores",label = " ", value = 5,min = 0))
                  ),
                  verbatimTextOutput("muestra"),br()

        )
      )
    )
  })
  
  ########################### NORMAL -- COMPONENTE 3 ##########################################################
  
  output$component3 <- renderUI({
    sidebarLayout(
      sidebarPanel(
        wellPanel(style= "background:grey",
                  h3("Distribución normal"),
                  h4(em("Parámetros de la distrubución normal: ")),
                  numericInput(inputId = "mu", label = "Media", value = 0),
                  numericInput(inputId = "sigma",label = "Desviación Típica", value = 1),
                  br(),
                  img(src = "normal.gif", height = 200, width = 200),
                  br(),br()
        )
        
      ),
      mainPanel(
        wellPanel(style= "background:grey",
                  h3("La distribución normal"),br(),
                  
                  strong("Representación de la función de densidad "),br(),br(),
                  plotOutput("graficoNormal"),
                  
                  fluidRow(column(6,br(),strong("Probabilidad de que la variable aleatoria sea menor que ")),
                           column(2, numericInput(inputId = "menor",label = " ", value = 1, step = 0.1))
                  ),
                  verbatimTextOutput("Pmenor"),br(),
                  plotOutput("graficoNormalMenor"),br(),
                  
                  fluidRow(column(6,br(),strong("Probabilidad de que la variable aleatoria sea mayor que ")),
                           column(2, numericInput(inputId = "mayor",label = " ", value =0,step = 0.1))
                  ),
                  verbatimTextOutput("Pmayor"),br(),
                  plotOutput("graficoNormalMayor"),br(),
                  
                  fluidRow(column(5,br(),strong("Probabilidad de que la variable aleatoria se encuentre entre ")),
                           column(2, numericInput(inputId = "menorI",label = " ", value = 0, step = 0.1)),
                           column(1,br(),strong(" y")),
                           column(2, numericInput(inputId = "mayorI",label = " ", value = 1,step = 0.1))
                  ),
                  verbatimTextOutput("probabilidadIntervaloN"),br(),
                  strong("Representación de la probabilidad calculada anteriormente"),br(),br(),
                  plotOutput("graficoNormalProbabilidad"),br(),
                  
                  fluidRow(column(7,br(),strong("Valor de la variable que deja a su derecha una probabilidad de ")),
                           column(2, numericInput(inputId = "porcentajeN",label = " ", value = 0.1,min = 0, max = 1, step = 0.05))
                  ),
                  verbatimTextOutput("PorcentajeObsN"),br(),
                  
                  fluidRow(column(8,br(),strong("Generación de una muestra aleatoria de tamaño ")),
                           column(2, numericInput(inputId = "valoresN",label = " ", value = 5,min = 0))
                  ),
                  verbatimTextOutput("muestraN")
                  
        )
      )
    )
  })
  
  ########################### APROXIMACION -- COMPONENTE 4 ##########################################################
  
  output$component4 <- renderUI({
    sidebarLayout(
      sidebarPanel(
        wellPanel(style= "background:grey",
                  h4("Aproximación de la distribución binomial a la Normal"),
                  p("Parámetros de la distrubución binomial: "),
                  numericInput(inputId = "n1", label = "Número de repeticiones, n", min = 0, value = 50),
                  numericInput(inputId = "p1",label = "Probabilidad de éxito, p",min = 0,max = 1, value = 0.5,step = 0.05),
                  br(),img(src = "dado.gif", height = 200, width = 200),br(),br()
        )
      ),
      mainPanel(
        wellPanel(style= "background:grey",
                  h4("Aproximación de la distribución binomial a la Normal"),br(),
                  strong("Gráfico de aproximación"),br(),br(),
                  textOutput("NoAprox"),br(),
                  plotOutput("graficoBinomialNormal")
        )
      )
    )
  })
  
  ########################### PROBABILIDAD -- COMPONENTE 1 ##########################################################
  output$graficoDados <- renderPlot({
    dice(rolls = input$rolls, ndice = input$ndice ,plot.it = T)
  })
  
  output$mediaDados <- renderPrint({
    tmp <- dice(rolls = input$rolls, ndice = input$ndice)
    colMeans(tmp)
  })
  
  ########################### BINOMIAL -- COMPONENTE 2 ########################################################## 
  
  output$esperanza <- renderPrint({
    cat(input$n*input$p)
  })
  
  output$varianza <- renderPrint({
    cat(input$n*input$p*(1-input$p))
  })
  
  output$probabilidad <- renderPrint({
    cat(dbinom(input$exitos,input$n,input$p))
  })
  
  output$probabilidadAlMenos <- renderPrint({
    cat(1-pbinom(input$exitos1-1,input$n,input$p))
  })
  
  output$probabilidadMucho <- renderPrint({
    cat(pbinom(input$exitos2-1,input$n,input$p,lower.tail = T))
  })
  
  output$probabilidadIntervalo <- renderPrint({
    if(input$exitosS >= input$exitosP){
      IC <- pbinom(c(input$exitosP-1,input$exitosS),input$n,input$p)
      cat(IC[2]-IC[1])
    }else{NULL}
    
  })
  
  output$PorcentajeObs <- renderPrint({
    cat(qbinom(1-input$porcentaje, input$n,input$p))
  })
  
  output$muestra <- renderPrint({
    cat(rbinom(input$valores,input$n,input$p))
  })
  
  output$graficoBinomial <- renderPlot({
    plot(dbinom(0:input$n,input$n,input$p),type = "h",xlab="x",ylab="P(X=x)",main=expression(paste("Funcion de probabilidad B(", n, ",",p,")")))
  })
  
  
  ########################### NORMAL -- COMPONENTE 3 ############################################################
  output$Pmenor <- renderPrint({
    cat(pnorm(input$menor,input$mu,input$sigma))
  })
  
  output$graficoNormalMenor <- renderPlot({
    curve(dnorm(x,input$mu,input$sigma),input$mu-(input$mu+10)/2,input$mu+(input$mu+10)/2,lwd=2, ylab = "f(x)",main=expression(paste("Densidad de la Normal N(", mu, ",", sigma^2,")")))
    t<-seq(input$mu-(input$mu+10)/2,input$menor,by=0.01)
    x <- c(-100000000000,t,input$menor)
    y <- c(0,dnorm(t,input$mu,input$sigma),0) 
    polygon(x,y,col="orange1")
  })
  
  output$Pmayor <- renderPrint({
    cat(1-pnorm(input$mayor,input$mu,input$sigma))
  })
  
  output$probabilidadIntervaloN <- renderPrint({
    if(input$mayorI >= input$menorI){
      IC <- pnorm(c(input$mayorI,input$menorI),input$mu,input$sigma)
      cat(IC[1]-IC[2])
    }else{NULL}
    
  })
  
  output$graficoNormalProbabilidad <- renderPlot({
    if(input$menorI <=input$mayorI){
      curve(dnorm(x,input$mu,input$sigma),input$mu-(input$mu+10)/2,input$mu+(input$mu+10)/2,lwd=2, ylab = "f(x)",main=expression(paste("Densidad de la Normal N(", mu, ",", sigma^2,")")))
      t<-seq(input$menorI,input$mayorI,by=0.01)
      x <- c(input$menorI,t,input$mayorI)
      y <- c(0,dnorm(t,input$mu,input$sigma),0) 
      polygon(x,y,col="orange1")
    }else{NULL}
  })
  
  output$graficoNormalMayor <- renderPlot({
    curve(dnorm(x,input$mu,input$sigma),input$mu-(input$mu+10)/2,input$mu+(input$mu+10)/2,lwd=2, ylab = "f(x)",main=expression(paste("Densidad de la Normal N(", mu, ",", sigma^2,")")))
    t<-seq(input$mayor,input$mu+(input$mu+10)/2,by=0.01)
    x <- c(input$mayor,t,100000000000)
    y <- c(0,dnorm(t,input$mu,input$sigma),0) 
    polygon(x,y,col="orange1")
  })
  
  output$PorcentajeObsN <- renderPrint({
    cat(qnorm(1-input$porcentajeN, input$mu,input$sigma))
  })
  
  output$muestraN <- renderPrint({
    cat(rnorm(input$valoresN,input$mu,input$sigma))
  })
  
  output$graficoNormal <- renderPlot({
    x <- seq(1,1)
    a <- input$mu
    curve(dnorm(x, input$mu, input$sigma),xlim=c(input$mu-(input$mu+10)/2,input$mu+(input$mu+10)/2),col="blue",lwd=2,xlab="x",ylab="f(x)",main=expression(paste("Densidad de la Normal N(", mu, ",", sigma^2,")")))
  })
  
  ########################### APROXIMACION BINOMIAL A NORMAL -- COMPONENTE 4 ####################################
  output$NoAprox <- renderText({
    if(input$n1*input$p1 < 3){
      "Esta binomial no aproxima bien a la normal. Modifica el valor de las repeticiones, n, o de la probabilidad, p"
    }  
  })
  
  output$graficoBinomialNormal <- renderPlot({
      plot(dbinom(0:(input$n1+input$n1/2),input$n1,input$p1),type = "h",xlab="x",ylab="P(X=x)")
      lines(dnorm(0:(input$n1+input$n1/2),input$n1*input$p1,sqrt(input$n1*input$p1*(1-input$p1))),col=2)
  })
  
  
}
shinyApp(ui = ui, server = server)