# install.packages("shiny", dependencies = T)
# install.packages("shinythemes", dependencies = T)
# install.packages("agricolae", dependencies = T)

library(shiny)
library(shinythemes)
library(agricolae)

Mode <- function(x) {
  ux <- unique(x)
  ux[which.max(tabulate(match(x, ux)))]
}


ui <- shinyUI(navbarPage(theme = shinytheme("united"),"DESCRIPCIÓN DE 1 VARIABLE",
                         tabPanel("Elección archivo",uiOutput("component1")),
                         tabPanel("Elección variable",uiOutput("component2")),
                         tabPanel("Tabla Frecuencias", uiOutput("component3")),
                         tabPanel("Estadísticos", uiOutput("component4")),
                         tabPanel("Gráficos", uiOutput("component5"))
                         
))

server <- function(input, output) {
  
  output$component1 <- renderUI({
    sidebarLayout(
      sidebarPanel(
        fileInput('file1', 'Lectura de fichero: ',
                  accept = c(
                    'text/csv',
                    'text/comma-separated-values',
                    'text/tab-separated-values',
                    'text/plain',
                    '.csv'
                  ),
                  buttonLabel = "Buscar fichero",
                  placeholder = "Ningún archivo seleccionado"
        ),
        radioButtons("header",
                     label="¿El archivo contiene el nombre de la variable?",
                     choices=list("Sí","No"),
                     selected="Sí"),
        radioButtons(inputId = 'sep', label='Separador de variables:',
                     c(Espacio ='\ ',
                       "Tabulación"='\t',
                       Coma=',',
                       "Punto y Coma"=';'
                       ),
                     '\ '),
        radioButtons('quote', 'Formato:',
                     c('Comilla Doble'='"',
                       'Comilla simple'="'",
                       "Nada"=''),
                     '"'),
        numericInput(inputId = "obs",label = "Número de filas a mostrar:",value=10),
        tags$hr()
      ),
      mainPanel(
        fluidRow(column(8,
                        h1("Fichero de datos"),
                        h4("Para variables cualitativas se obtiene: "),
                        p("Tabla de frecuencias",align = "center"),
                        p("Gráficos",align = "center"),
                        h4("Además, para variables numéricas se obtiene: "),
                        p("Estadísticos de centro o localización",align = "center"),
                        p("Estadísticos de dispersión",align = "center"),
                        br(),br(),
                        h4("Las dimensiones del fichero escogido son:"),
                        fluidRow(column(2,br()),
                                 column(6,p("Número de observaciones:")),
                                 column(4,verbatimTextOutput("dimensionesFilas",placeholder = T))
                        ),
                        fluidRow(column(2,br()),
                                 column(6,p("Número de variables:")),
                                 column(4,verbatimTextOutput("dimensionesColumnas",placeholder = T))
                        ),
                        h4("Primeros datos del fichero:"),
                        tableOutput('contents')
        ),
        column(4, br(),br(),img(src = "numeros.gif", height = 225, width = 225))
        )
      )
    )
  })
  
  output$component2 <- renderUI({
    sidebarLayout(
      sidebarPanel(
        h4("Elige la variable de estudio del fichero"),
        numericInput(inputId = "var",label = "Variable escogida:",value=1),
        radioButtons("tipo",
                     label="Selecciona si la variable es numérica o cualitativa:",
                     choices=list("Cualitativa","Numérica"),
                     selected="Cualitativa"),
        conditionalPanel(condition = "input.tipo == 'Numérica'",
          radioButtons("tipoNumerica",
                       label="Escoge el tipo de numérica:",
                       choices=list("Continua","Discreta"),
                       selected="Continua")
        ),
        numericInput(inputId = "obs1",label = "Número de observaciones a mostrar:",value=10)
        
      ),
      mainPanel(
        fluidRow(column(10,fluidRow(style="border:1px inset;border-color:#458cc3;background-color:#FA7457",
                                    column(5,h3("Primeros datos de la variable:")),
                                    column(5,br(),verbatimTextOutput("nombre")))),
                 column(1,br())
                 
        ),br(),
        tableOutput('contentsV')
      )
    )
  })
  
  output$component3 <- renderUI({
    sidebarLayout(
      sidebarPanel(width = 3,
        h3("Tabla de frecuencias"),
        p("Frecuencia absoluta (fi)"),
        p("Frecuencia relativa (fr)"),
        p("Porcentaje: Frecuencia relativa*100"),br(),
        strong(h4(em("Si el tipo de variable es declarado erróneamente puede que los resultados no sean válidos."))),br()
      ),
      mainPanel(
        fluidRow(column(10,fluidRow(style="border:1px inset;border-color:#458cc3;background-color:#FA7457",
                                    column(7,h3("Cálculo de tabla de frecuencias para la variable:")),
                                    column(3,br(),verbatimTextOutput("nombre1")))),
                 column(1,br()))
        ,br(),
        conditionalPanel(condition = "input.tipo =='Numérica' && input.tipoNumerica == 'Continua'",
                         p("Escoge el número de intervalos en los que agrupar los datos:"),
                         numericInput(inputId = "particion",label = " ",value=10)
                         ),
        tableOutput("view")
      )
    )
  })
  
  output$component4 <- renderUI({
    sidebarLayout(
      sidebarPanel(width = 3,
        h3("Estadísticos de centro: "),
        strong("Mínimo y Máximo"),br(),
        strong("Cuartiles"),br(),
        strong("Percentil"),br(),
        strong("Media aritmética"),br(),
        strong("Mediana"),br(),
        strong("Moda"),br(),
        h3("Estadísticos de dispersión: "),
        strong("Rango o recorrido"),
        strong("Varianza"),br(),
        strong("Desviación típica"),br(),
        strong("Coeficiente de variación"),br(),
        strong("Rango intercuartilico"),br(),br(),
        strong(h4(em("Si el tipo de variable es declarado erróneamente los resultados puede que no sean válidos"))),br()
      ),
      mainPanel(
        conditionalPanel(condition = "input.tipo == 'Numérica'",
           fluidRow(column(10,fluidRow(style="border:1px inset;border-color:#458cc3;background-color:#FA7457",
                                       column(7,h3("Cálculo de estadísticos para la variable:")),
                                       column(3,br(),verbatimTextOutput("nombre2")))),
                   column(1,br()))        
          ,br(),
          fluidRow(
            column(5,
                   p("Mínimo y máximo:"),
                   verbatimTextOutput("minmax"),
                   p("Primer y tercer cuartil:"),
                   verbatimTextOutput("cuartiles"),
                   fluidRow(column(4,br(),p("Percentil:")),
                            column(4,numericInput(inputId = "perc",label = " ",value=20))),
                   verbatimTextOutput("percentil"),
                   p("Media:"),
                   verbatimTextOutput("media"),
                   p("Mediana o segundo cuartil:"),
                   verbatimTextOutput("mediana"),
                   p("Moda:"),
                   verbatimTextOutput("moda")
            ),
            column(5,
                   p("Rango o recorrido:"),
                   verbatimTextOutput("rango"),
                   p("Varianza:"),
                   verbatimTextOutput("varianza"),
                   p("Desviación típica: "),
                   verbatimTextOutput("desviacion"),
                   p("Coeficiente de variacion: "),
                   verbatimTextOutput("cv"),
                   p("Rango intercuartilico: "),
                   verbatimTextOutput("rangoIntercuartilico"))
          )
        ),
        conditionalPanel(condition = "input.tipo == 'Cualitativa'",
                         h4("Para obtener resultados en esta sección la variable tiene que ser declarada numérica"))  
        ))
  })
  
  output$component5 <- renderUI({
    sidebarLayout(
      sidebarPanel(width=3,
        wellPanel(style= "background:white",
                  img(src = "sectores.gif", height = 150, width = 200), br(),
                  h5("Gráfico de barras"),
                  h5("Gráfico de sectores"),
                  h5("Diagrama de cajas y bigotes"),
                  h5("Histograma"),
                  h5("Polígono de frecuencias"),
                  h4(strong(em("Si se ha declarado erróneamente el tipo de variable los resultados puede que no sean válidos"))),br(),br(),
                  h6("Dependiendo del tipo de variable declarado se dibujaran unos gráficos u otros"),
                  h6("Para variables numéricas continuas el diagrama de varillas y sectores se dibujara cuando n < 15")
        )
        
      ),
      mainPanel(
        fluidRow(column(10,fluidRow(style="border:1px inset;border-color:#458cc3;background-color:#FA7457",
                                    column(7,h3("Gráficos para la variable:")),
                                    column(3,br(),verbatimTextOutput("nombre3")))),
                 column(1,br())
        ),
        conditionalPanel(condition = "input.tipo == 'Cualitativa'",
            h4("Gráfico de barras"),
            plotOutput(outputId = "barras"),
            h4("Gráfico de sectores"),
            plotOutput(outputId = "sectores")
        ),
        conditionalPanel(condition = "input.tipo == 'Numérica' && input.tipoNumerica == 'Discreta'",
                         h4("Gráfico de barras"),
                         plotOutput(outputId = "barrasD"),
                         h4("Gráfico de sectores"),
                         plotOutput(outputId = "sectoresD")
        ),
        conditionalPanel(condition = "input.tipo == 'Numérica' && input.tipoNumerica == 'Continua'",
                         conditionalPanel(condition = "'length(unique(datasetInput1()[,1]))' < 16",
                                          h4("Gráfico de barras"),
                                          plotOutput(outputId = "barrasC"),
                                          h4("Gráfico de sectores"),
                                          plotOutput(outputId = "sectoresC")
                        ),
                         h4("Cajas y Bigotes"),
                         plotOutput(outputId = "cajas"), 
                         p("Para los siguientes gráficos podemos elegir el número de particiones: "),
                         sliderInput(inputId = "bins",label = " ",min = 1,max=100,value=5),
                         h4("Histograma"),
                         plotOutput(outputId = "histograma"),
                         p("Particiones del histograma:"),
                         verbatimTextOutput("clases"),
                         h4("Polígono de frecuencias"),
                         plotOutput(outputId = "polFrec"),
                         plotOutput(outputId = "polFrecSIN")
        )

      )
    )
  })
  
  ########################### LECTURA DE DATOS -- cOMPONENTE 1 ####################################  
  
  datasetInput <- reactive({
    req(input$file1)
    inFile <- input$file1
    if(input$header == "Sí"){
      a <- TRUE
    }else{ a <- FALSE}
    db <- read.csv(inFile$datapath, header = a,
                   sep = input$sep, quote = input$quote)
    db <- data.frame(db)
  })
  
  datasetInput1 <- reactive({
    db <- datasetInput()
    if(input$var <= 0){
      db <- NULL
    }else if(input$var > ncol(db)){
      db <- NULL
    }else{
      nombre <- names(datasetInput())
      db <-as.data.frame(db[,input$var])
      colnames(db) <- nombre[input$var]
      db
    }
  })
  
  output$contents <- renderTable({
    if(input$obs <= 0){
      NULL
    }else{
      head(datasetInput(),n=input$obs)
    }
  })
  
  output$dimensionesFilas <- renderPrint({
      cat(nrow(datasetInput()))
  }) 
  
  output$dimensionesColumnas <- renderPrint({
    cat(ncol(datasetInput()))
  })
  
  output$contentsV <- renderTable({
    if(input$obs <=0){
      NULL
    }else{
      head(datasetInput1(),n=input$obs1)
    }
  })
  
  output$nombre <- renderPrint({
    cat(names(datasetInput1())[1])
  }) 
  
  output$nombre1 <- renderPrint({
    cat(names(datasetInput1())[1])
  }) 
  output$nombre2 <- renderPrint({
    cat(names(datasetInput1())[1])
  }) 
  output$nombre3 <- renderPrint({
    cat(names(datasetInput1())[1])
  }) 
  
  ########################### TABLA DE FRECUENCIAS -- COMPONENTE 2 #################################### 
  output$particionIntervalo <- renderText({
    if(input$tipo=="Numérica" && input$tipoNumerica =="Continua"){
      "Escoger número de intervalos:"
    }  
  })
  
  output$view <- renderTable({
    if(input$tipo == "Cualitativa"){
      fi <- table(datasetInput()[,input$var])
      fr <- round(prop.table(table(datasetInput()[,input$var])),3)
      Porcentaje <- round(fr*100,4)
      tabla <- data.frame(fi,fr,Porcentaje)
      tabla <- tabla[,-c(3,5)]
      colnames(tabla) <- c("Datos","Frecuencia absoluta","Frecuencia relativa","Porcentaje")
      tabla
    }
    else if(input$tipo == "Numérica" && input$tipoNumerica == "Discreta"){
      fi <- table(datasetInput()[,input$var])
      fr <- round(prop.table(table(datasetInput()[,input$var])),3)
      Porcentaje <- round(fr*100,4)
      tabla <- data.frame(fi,fr,Porcentaje)
      tabla <- tabla[,-c(3,5)]
      colnames(tabla) <- c("Datos","Frecuencia absoluta","Frecuencia relativa","Porcentaje")
      tabla
    }else if(input$tipo == "Numérica" && input$tipoNumerica == "Continua"){
      a <- factor(cut(datasetInput()[,input$var],breaks = input$particion))
      fi <- (table(a))
      fr <- round(prop.table(table(a)),3)
      Porcentaje <- round(fr*100,4)
      tabla <- data.frame(fi,fr,Porcentaje)
      tabla <- tabla[,-c(3,5)]
      colnames(tabla) <- c("Datos","Frecuencia absoluta","Frecuencia relativa","Porcentaje")
      tabla
    }
    
  })
  
  ########################### ESTADISTICOS -- COMPONENTE 3 #################################### 
  
  output$minmax <- renderPrint({
      summary(datasetInput1()[,1])[c(1,6)]
  }) 
  
  output$cuartiles <- renderPrint({
      summary(datasetInput1()[,1])[c(2,5)]
  })  
  
  output$percentil <- renderPrint({
      quantile( datasetInput1()[,1] , prob  = input$perc/100)
  })  
  
  output$media <- renderPrint({
      cat(summary(datasetInput1()[,1])[4])
  })
  
  output$mediana <- renderPrint({
      cat(summary(datasetInput1()[,1])[3])
  })
  
  output$moda <- renderPrint({
      cat(Mode(datasetInput1()[,1]))
  })
  
  output$rango <- renderPrint({
      cat(max(datasetInput1()[,1])-min(datasetInput1()[,1]))
  })
  
  output$varianza <- renderPrint({
      cat(var(datasetInput1()[,1]))
  })
  
  output$desviacion <- renderPrint({
      cat(sd(datasetInput1()[,1]))
  })
  
  output$cv <- renderPrint({
      CV <- sd(datasetInput1()[,1])/mean(datasetInput1()[,1])
      CVp <- CV*100
      tabla <- cbind(CV,CVp)
      colnames(tabla) <- c("CV","CV %")
      rownames(tabla) <- c("")
      tabla
  })
  
  output$rangoIntercuartilico <- renderPrint({
      cat(IQR(datasetInput1()[,1]))
  })
  
  ########################### GRAFICOS -- COMPONENTE 4 #################################### 
  
  output$barras <- renderPlot({
      barplot((table(datasetInput1()[,1])), col = "blue",border = NA, main = names(datasetInput1())[1])
  })
  
  output$barrasD <- renderPlot({
      plot((table(datasetInput1()[,1])), type = "h",col = "green", cex = 4, ylab = " ", main = names(datasetInput1())[1])
  })
  
  output$barrasC <- renderPlot({
        plot((table(datasetInput1()[,1])), type = "h",col = "pink", cex = 4, ylab = " ", main = names(datasetInput1())[1])
  })
  
  output$sectores <- renderPlot({
      pie(prop.table(table(datasetInput1()[,1])), main = names(datasetInput1())[1])
  })
  
  output$sectoresD <- renderPlot({
      pie(prop.table(table(datasetInput1()[,1])), main = names(datasetInput1())[1])
  })
  
  output$sectoresC <- renderPlot({
        pie(prop.table(table(datasetInput1()[,1])), main = names(datasetInput1())[1])
  })
  
  output$cajas <- renderPlot({
      boxplot(datasetInput1()[,1],col="red",main = names(datasetInput1())[1])
  })
  
  output$histograma <- renderPlot({
      bins <- seq(min(datasetInput1()[,1]),max(datasetInput1()[,1]),length.out = input$bins+1)
      hist(datasetInput1()[,1],breaks = bins,col = "darkslategray2",xlab = "intervalos",ylab="Frecuencias absolutas", main = names(datasetInput1())[1])
  })
  
  output$clases <- renderPrint({
      bins <- seq(min(datasetInput1()[,1]),max(datasetInput1()[,1]),length.out = input$bins+1)
      cat(bins)
  })
  
  output$polFrec <- renderPlot({
      bins <- seq(min(datasetInput1()[,1]),max(datasetInput1()[,1]),length.out = input$bins+1)
      h <- hist(datasetInput1()[,1],breaks = bins,col = "darkslategray2",xlab = "intervalos",ylab="Frecuencias absolutas", main = names(datasetInput1())[1])
      polygon.freq(h,col="red",frequency=1,lwd=2)
  })
  output$polFrecSIN <- renderPlot({
      bins <- seq(min(datasetInput1()[,1]),max(datasetInput1()[,1]),length.out = input$bins+1)
      h <- hist(datasetInput1()[,1],breaks = bins,col = "white",border = "white",xlab = "intervalos",ylab="Frecuencias absolutas", main = names(datasetInput1())[1])
      polygon.freq(h,col="red",frequency=1,lwd=2)
  })
  
}

###################################################################

shinyApp(ui = ui, server = server)
